@@ -1,4 +1,5 @@
-"""The Textual command palette.
+"""
+This module contains classes for working with Textual's command palette.
 
 See the guide on the [Command Palette](../guide/command_palette.md) for full details.
 
@@ -19,32 +20,44 @@ from asyncio import (
 from dataclasses import dataclass
 from functools import total_ordering
 from inspect import isclass
+from operator import attrgetter
 from time import monotonic
-from typing import TYPE_CHECKING, Any, AsyncGenerator, AsyncIterator, ClassVar, Iterable
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    AsyncGenerator,
+    AsyncIterator,
+    Callable,
+    ClassVar,
+    Iterable,
+    NamedTuple,
+)
 
 import rich.repr
 from rich.align import Align
-from rich.console import Group, RenderableType
-from rich.style import Style
 from rich.text import Text
 from typing_extensions import Final, TypeAlias
 
-from . import on, work
-from .binding import Binding, BindingType
-from .containers import Horizontal, Vertical
-from .events import Click, Mount
-from .fuzzy import Matcher
-from .reactive import var
-from .screen import Screen, _SystemModalScreen
-from .timer import Timer
-from .types import CallbackType, IgnoreReturnCallbackType
-from .widget import Widget
-from .widgets import Button, Input, LoadingIndicator, OptionList, Static
-from .widgets.option_list import Option
-from .worker import get_current_worker
+from textual import on, work
+from textual.binding import Binding, BindingType
+from textual.containers import Horizontal, Vertical
+from textual.content import Content
+from textual.events import Click, Mount
+from textual.fuzzy import Matcher
+from textual.message import Message
+from textual.reactive import var
+from textual.screen import Screen, SystemModalScreen
+from textual.style import Style
+from textual.timer import Timer
+from textual.types import IgnoreReturnCallbackType
+from textual.visual import VisualType
+from textual.widget import Widget
+from textual.widgets import Button, Input, LoadingIndicator, OptionList, Static
+from textual.widgets.option_list import Option
+from textual.worker import get_current_worker
 
 if TYPE_CHECKING:
-    from .app import App, ComposeResult
+    from textual.app import App, ComposeResult
 
 __all__ = [
     "CommandPalette",
@@ -66,7 +79,7 @@ class Hit:
     The value should be between 0 (no match) and 1 (complete match).
     """
 
-    match_display: RenderableType
+    match_display: VisualType
     """A string or Rich renderable representation of the hit."""
 
     command: IgnoreReturnCallbackType
@@ -83,7 +96,7 @@ class Hit:
     """Optional help text for the command."""
 
     @property
-    def prompt(self) -> RenderableType:
+    def prompt(self) -> VisualType:
         """The prompt to use when displaying the hit in the command palette."""
         return self.match_display
 
@@ -100,21 +113,14 @@ class Hit:
     def __post_init__(self) -> None:
         """Ensure 'text' is populated."""
         if self.text is None:
-            if isinstance(self.match_display, str):
-                self.text = self.match_display
-            elif isinstance(self.match_display, Text):
-                self.text = self.match_display.plain
-            else:
-                raise ValueError(
-                    "A value for 'text' is required if 'match_display' is not a str or Text"
-                )
+            self.text = str(self.match_display)
 
 
 @dataclass
 class DiscoveryHit:
     """Holds the details of a single command search hit."""
 
-    display: RenderableType
+    display: VisualType
     """A string or Rich renderable representation of the hit."""
 
     command: IgnoreReturnCallbackType
@@ -131,10 +137,20 @@ class DiscoveryHit:
     """Optional help text for the command."""
 
     @property
-    def prompt(self) -> RenderableType:
+    def prompt(self) -> VisualType:
         """The prompt to use when displaying the discovery hit in the command palette."""
         return self.display
 
+    @property
+    def score(self) -> float:
+        """A discovery hit always has a score of 0.
+
+        The order in which discovery hits are displayed is determined by the order
+        in which they are yielded by the Provider. It's up to the developer to yield
+        DiscoveryHits in the .
+        """
+        return 0.0
+
     def __lt__(self, other: object) -> bool:
         if isinstance(other, DiscoveryHit):
             assert self.text is not None
@@ -150,19 +166,15 @@ class DiscoveryHit:
     def __post_init__(self) -> None:
         """Ensure 'text' is populated."""
         if self.text is None:
-            if isinstance(self.display, str):
-                self.text = self.display
-            elif isinstance(self.display, Text):
-                self.text = self.display.plain
-            else:
-                raise ValueError(
-                    "A value for 'text' is required if 'display' is not a str or Text"
-                )
+            self.text = str(self.display)
 
 
 Hits: TypeAlias = AsyncIterator["DiscoveryHit | Hit"]
 """Return type for the command provider's `search` method."""
 
+ProviderSource: TypeAlias = "Iterable[type[Provider] | Callable[[], type[Provider]]]"
+"""The type used to declare the providers for a CommandPalette."""
+
 
 class Provider(ABC):
     """Base class for command palette command providers.
@@ -177,6 +189,10 @@ class Provider(ABC):
         Args:
             screen: A reference to the active screen.
         """
+        if match_style is not None:
+            assert isinstance(
+                match_style, Style
+            ), "match_style must be a Visual style (from textual.style import Style)"
         self.__screen = screen
         self.__match_style = match_style
         self._init_task: Task | None = None
@@ -216,7 +232,9 @@ class Provider(ABC):
             A [fuzzy matcher][textual.fuzzy.Matcher] object for matching against candidate hits.
         """
         return Matcher(
-            user_input, match_style=self.match_style, case_sensitive=case_sensitive
+            user_input,
+            match_style=self.match_style,
+            case_sensitive=case_sensitive,
         )
 
     def _post_init(self) -> None:
@@ -239,6 +257,7 @@ class Provider(ABC):
         """Wait for initialization."""
         if self._init_task is not None:
             await self._init_task
+        self._init_task = None
 
     async def startup(self) -> None:
         """Called after the Provider is initialized, but before any calls to `search`."""
@@ -306,15 +325,87 @@ class Provider(ABC):
         """
 
 
-@rich.repr.auto
-@total_ordering
-class Command(Option):
-    """Class that holds a command in the [`CommandList`][textual.command.CommandList]."""
+class SimpleCommand(NamedTuple):
+    """A simple command."""
+
+    name: str
+    """The name of the command."""
+    callback: IgnoreReturnCallbackType
+    """The callback to invoke when the command is selected."""
+    help_text: str | None = None
+    """The description of the command."""
+
+
+CommandListItem: TypeAlias = (
+    "SimpleCommand | tuple[str, IgnoreReturnCallbackType, str | None] | tuple[str, IgnoreReturnCallbackType]"
+)
+
+
+class SimpleProvider(Provider):
+    """A simple provider which the caller can pass commands to."""
 
     def __init__(
         self,
-        prompt: RenderableType,
-        command: DiscoveryHit | Hit,
+        screen: Screen[Any],
+        commands: list[CommandListItem],
+    ) -> None:
+        # Convert all commands to SimpleCommand instances
+        super().__init__(screen, None)
+        self._commands: list[SimpleCommand] = []
+        for command in commands:
+            if isinstance(command, SimpleCommand):
+                self._commands.append(command)
+            elif len(command) == 2:
+                self._commands.append(SimpleCommand(*command, None))
+            elif len(command) == 3:
+                self._commands.append(SimpleCommand(*command))
+            else:
+                raise ValueError(f"Invalid command: {command}")
+
+    def __call__(
+        self, screen: Screen[Any], match_style: Style | None = None
+    ) -> SimpleProvider:
+        self.__match_style = match_style
+        return self
+
+    @property
+    def match_style(self) -> Style | None:
+        return self.__match_style
+
+    async def search(self, query: str) -> Hits:
+        matcher = self.matcher(query)
+        for name, callback, help_text in self._commands:
+            if (match := matcher.match(name)) > 0:
+                yield Hit(
+                    match,
+                    matcher.highlight(name),
+                    callback,
+                    help=help_text,
+                )
+
+    async def discover(self) -> Hits:
+        """Handle a request for the discovery commands for this provider.
+
+        Yields:
+            Commands that can be discovered.
+        """
+        for name, callback, help_text in self._commands:
+            yield DiscoveryHit(
+                name,
+                callback,
+                help=help_text,
+            )
+
+
+@rich.repr.auto
+@total_ordering
+class Command(Option):
+    """Class that holds a hit in the [`CommandList`][textual.command.CommandList]."""
+
+    def __init__(
+        self,
+        prompt: VisualType,
+        hit: DiscoveryHit | Hit,
         id: str | None = None,
         disabled: bool = False,
     ) -> None:
@@ -322,22 +413,25 @@ class Command(Option):
 
         Args:
             prompt: The prompt for the option.
-            command: The details of the command associated with the option.
+            hit: The details of the hit associated with the option.
             id: The optional ID for the option.
             disabled: The initial enabled/disabled state. Enabled by default.
         """
         super().__init__(prompt, id, disabled)
-        self.command = command
-        """The details of the command associated with the option."""
+        self.hit = hit
+        """The details of the hit associated with the option."""
+
+    def __hash__(self) -> int:
+        return id(self)
 
     def __lt__(self, other: object) -> bool:
         if isinstance(other, Command):
-            return self.command < other.command
+            return self.hit < other.hit
         return NotImplemented
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, Command):
-            return self.command == other.command
+            return self.hit == other.hit
         return NotImplemented
 
 
@@ -348,12 +442,13 @@ class CommandList(OptionList, can_focus=False):
     CommandList {
         visibility: hidden;
         border-top: blank;
-        border-bottom: hkey $primary;
+        border-bottom: hkey black;
         border-left: none;
         border-right: none;
         height: auto;
         max-height: 70vh;
-        background: $panel;
+        background: transparent;
+        padding: 0;
     }
 
     CommandList:focus {
@@ -369,11 +464,19 @@ class CommandList(OptionList, can_focus=False):
     }
 
     CommandList > .option-list--option-highlighted {
-        background: $accent;
+        color: $block-cursor-blurred-foreground;
+        background: $block-cursor-blurred-background;
+        text-style: $block-cursor-blurred-text-style;
+    }
+
+    CommandList:nocolor > .option-list--option-highlighted {       
+        text-style: reverse;
     }
 
     CommandList > .option-list--option {
-        padding-left: 1;
+        padding: 0 2;
+        color: $foreground;
+        text-style: bold;
     }
     """
 
@@ -393,7 +496,7 @@ class SearchIcon(Static, inherit_css=False):
     icon: var[str] = var("🔎")
     """The icon to display."""
 
-    def render(self) -> RenderableType:
+    def render(self) -> VisualType:
         """Render the icon.
 
         Returns:
@@ -409,15 +512,18 @@ class CommandInput(Input):
     CommandInput, CommandInput:focus {
         border: blank;
         width: 1fr;
-        background: $panel;
         padding-left: 0;
+        background: transparent;
+        background-tint: 0%;
     }
     """
 
 
-class CommandPalette(_SystemModalScreen[CallbackType]):
+class CommandPalette(SystemModalScreen[None]):
     """The Textual command palette."""
 
+    AUTO_FOCUS = "CommandInput"
+
     COMPONENT_CLASSES: ClassVar[set[str]] = {
         "command-palette--help-text",
         "command-palette--highlight",
@@ -430,41 +536,57 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     """
 
     DEFAULT_CSS = """
+   
     CommandPalette:inline {
         /* If the command palette is invoked in inline mode, we may need additional lines. */
         min-height: 20;
     }
     CommandPalette {
-        background: $background 30%;
-        align-horizontal: center;
+        color: $foreground;
+        background: $background 60%;
+        align-horizontal: center;        
+
+        #--container {
+            display: none;
+        }
+
+        &:ansi {
+            background: transparent;
+        }
     }
 
-    CommandPalette > .command-palette--help-text {
-        background: transparent;
+    CommandPalette.-ready {
+        #--container {
+            display: block;
+        }
+    }
+
+    CommandPalette > .command-palette--help-text {                   
         color: $text-muted;
+        background: transparent;
+        text-style: not bold;       
     }
-
-    CommandPalette:dark > .command-palette--highlight {
-        text-style: bold;
-        color: $warning;
-    }
+    
     CommandPalette > .command-palette--highlight {
-        text-style: bold;
-        color: $warning-darken-2;
+        text-style: bold underline;
+    }   
+
+    CommandPalette:nocolor > .command-palette--highlight {
+        text-style: underline;
     }
 
     CommandPalette > Vertical {
-        margin-top: 3;
-        width: 90%;
+        margin-top: 3; 
         height: 100%;
         visibility: hidden;
+        background: $surface;
+        &:dark { background: $panel-darken-1; }
     }
 
     CommandPalette #--input {
         height: auto;
         visibility: visible;
-        border: hkey $primary;
-        background: $panel;
+        border: hkey black 50%;
     }
 
     CommandPalette #--input.--list-visible {
@@ -489,8 +611,7 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     CommandPalette LoadingIndicator {
         height: auto;
         visibility: hidden;
-        background: $panel;
-        border-bottom: hkey $primary;
+        border-bottom: hkey $border;
     }
 
     CommandPalette LoadingIndicator.--visible {
@@ -499,13 +620,23 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     """
 
     BINDINGS: ClassVar[list[BindingType]] = [
-        Binding("ctrl+end, shift+end", "command_list('last')", show=False),
-        Binding("ctrl+home, shift+home", "command_list('first')", show=False),
-        Binding("down", "cursor_down", show=False),
+        Binding(
+            "ctrl+end, shift+end",
+            "command_list('last')",
+            "Go to bottom",
+            show=False,
+        ),
+        Binding(
+            "ctrl+home, shift+home",
+            "command_list('first')",
+            "Go to top",
+            show=False,
+        ),
+        Binding("down", "cursor_down", "Next command", show=False),
         Binding("escape", "escape", "Exit the command palette"),
-        Binding("pagedown", "command_list('page_down')", show=False),
-        Binding("pageup", "command_list('page_up')", show=False),
-        Binding("up", "command_list('cursor_up')", show=False),
+        Binding("pagedown", "command_list('page_down')", "Next page", show=False),
+        Binding("pageup", "command_list('page_up')", "Previous page", show=False),
+        Binding("up", "command_list('cursor_up')", "Previous command", show=False),
     ]
     """
     | Key(s) | Description |
@@ -538,32 +669,71 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
     _calling_screen: var[Screen[Any] | None] = var(None)
     """A record of the screen that was active when we were called."""
 
-    _PALETTE_ID: Final[str] = "--command-palette"
-    """The internal ID for the command palette."""
+    @dataclass
+    class OptionHighlighted(Message):
+        """Posted to App when an option is highlighted in the command palette."""
+
+        highlighted_event: OptionList.OptionHighlighted
+        """The option highlighted event from the OptionList within the command palette."""
+
+    @dataclass
+    class Opened(Message):
+        """Posted to App when the command palette is opened."""
+
+    @dataclass
+    class Closed(Message):
+        """Posted to App when the command palette is closed."""
+
+        option_selected: bool
+        """True if an option was selected, False if the palette was closed without selecting an option."""
+
+    def __init__(
+        self,
+        providers: ProviderSource | None = None,
+        *,
+        placeholder: str = "Search for commands…",
+        name: str | None = None,
+        id: str | None = None,
+        classes: str | None = None,
+    ) -> None:
+        """Initialise the command palette.
+
+        Args:
+            providers: An optional list of providers to use. If None, the providers supplied
+                in the App or Screen will be used.
+            placeholder: The placeholder text for the command palette.
+        """
+        super().__init__(
+            id=id,
+            classes=classes,
+            name=name,
+        )
+        self.add_class("--textual-command-palette")
 
-    def __init__(self) -> None:
-        """Initialise the command palette."""
-        super().__init__(id=self._PALETTE_ID)
         self._selected_command: DiscoveryHit | Hit | None = None
         """The command that was selected by the user."""
         self._busy_timer: Timer | None = None
         """Keeps track of if there's a busy indication timer in effect."""
         self._no_matches_timer: Timer | None = None
         """Keeps track of if there are 'No matches found' message waiting to be displayed."""
+        self._supplied_providers: ProviderSource | None = providers
         self._providers: list[Provider] = []
         """List of Provider instances involved in searches."""
+        self._hit_count: int = 0
+        """Number of hits displayed."""
+        self._placeholder = placeholder
 
     @staticmethod
-    def is_open(app: App) -> bool:
-        """Is the command palette current open?
+    def is_open(app: App[object]) -> bool:
+        """Is a command palette current open?
 
         Args:
             app: The app to test.
 
         Returns:
-            `True` if the command palette is currently open, `False` if not.
+            `True` if a command palette is currently open, `False` if not.
         """
-        return app.screen.id == CommandPalette._PALETTE_ID
+        return app.screen.has_class("--textual-command-palette")
 
     @property
     def _provider_classes(self) -> set[type[Provider]]:
@@ -574,27 +744,36 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         the current screen][textual.screen.Screen.COMMANDS].
         """
 
-        def get_providers(root: App | Screen) -> Iterable[type[Provider]]:
-            """Get providers from app or screen.
+        def get_providers(
+            provider_source: ProviderSource,
+        ) -> Iterable[type[Provider]]:
+            """Load the providers from a source (typically from the COMMANDS class variable)
+            at the App or Screen level.
 
             Args:
-                root: The app or screen.
+                provider_source: The source of providers.
 
             Returns:
                 An iterable of providers.
             """
-            for provider in root.COMMANDS:
-                if isclass(provider) and issubclass(provider, Provider):
+            for provider in provider_source:
+                if isinstance(provider, SimpleProvider):
+                    yield provider
+                elif isclass(provider) and issubclass(provider, Provider):
                     yield provider
                 else:
                     # Lazy loaded providers
                     yield provider()  # type: ignore
 
-        return (
-            set()
-            if self._calling_screen is None
-            else {*get_providers(self.app), *get_providers(self._calling_screen)}
-        )
+        if self._calling_screen is None:
+            return set()
+        elif self._supplied_providers is None:
+            return {
+                *get_providers(self.app.COMMANDS),
+                *get_providers(self._calling_screen.COMMANDS),
+            }
+        else:
+            return {*get_providers(self._supplied_providers)}
 
     def compose(self) -> ComposeResult:
         """Compose the command palette.
@@ -602,17 +781,17 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         Returns:
             The content of the screen.
         """
-        with Vertical():
+        with Vertical(id="--container"):
             with Horizontal(id="--input"):
                 yield SearchIcon()
-                yield CommandInput(placeholder="Command Palette Search...")
+                yield CommandInput(placeholder=self._placeholder, select_on_focus=False)
                 if not self.run_on_select:
                     yield Button("\u25b6")
             with Vertical(id="--results"):
                 yield CommandList()
                 yield LoadingIndicator()
 
-    def _on_click(self, event: Click) -> None:
+    def _on_click(self, event: Click) -> None:  # type: ignore[override]
         """Handle the click event.
 
         Args:
@@ -623,15 +802,16 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         """
         if self.get_widget_at(event.screen_x, event.screen_y)[0] is self:
             self._cancel_gather_commands()
+            self.app.post_message(CommandPalette.Closed(option_selected=False))
             self.dismiss()
 
     def _on_mount(self, _: Mount) -> None:
         """Configure the command palette once the DOM is ready."""
+
+        self.app.post_message(CommandPalette.Opened())
         self._calling_screen = self.app.screen_stack[-2]
 
-        match_style = self.get_component_rich_style(
-            "command-palette--highlight", partial=True
-        )
+        match_style = self.get_visual_style("command-palette--highlight", partial=True)
 
         assert self._calling_screen is not None
         self._providers = [
@@ -642,7 +822,7 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             provider._post_init()
         self._gather_commands("")
 
-    async def _on_unmount(self) -> None:
+    async def _on_unmount(self) -> None:  # type: ignore[override]
         """Shutdown providers when command palette is closed."""
         if self._providers:
             await wait(
@@ -696,7 +876,7 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
                 command_list = self.query_one(CommandList)
                 command_list.add_option(
                     Option(
-                        Align.center(Text("No matches found")),
+                        Align.center(Text("No matches found", style="not bold")),
                         disabled=True,
                         id=self._NO_MATCHES,
                     )
@@ -769,7 +949,6 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             )
             for provider in self._providers
         ]
-
         # Set up a delay for showing that we're busy.
         self._start_busy_countdown()
 
@@ -831,41 +1010,6 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             for search in searches:
                 search.cancel()
 
-    @staticmethod
-    def _sans_background(style: Style) -> Style:
-        """Returns the given style minus the background color.
-
-        Args:
-            style: The style to remove the color from.
-
-        Returns:
-            The given style, minus its background.
-        """
-        # Here we're pulling out all of the styles *minus* the background.
-        # This should probably turn into a utility method on Style
-        # eventually. The reason for this is we want the developer to be
-        # able to style the help text with a component class, but we want
-        # the background to always be the background at any given moment in
-        # the context of an OptionList. At the moment this act of copying
-        # sans bgcolor seems to be the only way to achieve this.
-        return Style(
-            blink2=style.blink2,
-            blink=style.blink,
-            bold=style.bold,
-            color=style.color,
-            conceal=style.conceal,
-            dim=style.dim,
-            encircle=style.encircle,
-            frame=style.frame,
-            italic=style.italic,
-            link=style.link,
-            overline=style.overline,
-            reverse=style.reverse,
-            strike=style.strike,
-            underline2=style.underline2,
-            underline=style.underline,
-        )
-
     def _refresh_command_list(
         self, command_list: CommandList, commands: list[Command], clear_current: bool
     ) -> None:
@@ -876,22 +1020,15 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
             commands: The commands to show in the widget.
             clear_current: Should the current content of the list be cleared first?
         """
-        # For the moment, this is a fairly naive approach to populating the
-        # command list with a sorted list of commands. Every time we add a
-        # new one we're nuking the list of options and populating them
-        # again. If this turns out to not be a great approach, we may try
-        # and get a lot smarter with this (ideally OptionList will grow a
-        # method to sort its content in an efficient way; but for now we'll
-        # go with "worse is better" wisdom).
-        highlighted = (
-            command_list.get_option_at_index(command_list.highlighted)
-            if command_list.highlighted is not None and not clear_current
-            else None
-        )
-        command_list.clear_options().add_options(sorted(commands, reverse=True))
-        if highlighted is not None and highlighted.id:
-            command_list.highlighted = command_list.get_option_index(highlighted.id)
+
+        sorted_commands = sorted(commands, key=attrgetter("hit.score"), reverse=True)
+        command_list.clear_options().add_options(sorted_commands)
+
+        if sorted_commands:
+            command_list.highlighted = 0
+
         self._list_visible = bool(command_list.option_count)
+        self._hit_count = command_list.option_count
 
     _RESULT_BATCH_TIME: Final[float] = 0.25
     """How long to wait before adding commands to the command list."""
@@ -909,13 +1046,6 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         Args:
             search_value: The value to search for.
         """
-
-        # We'll potentially use the help text style a lot so let's grab it
-        # the once for use in the loop further down.
-        help_style = self._sans_background(
-            self.get_component_rich_style("command-palette--help-text")
-        )
-
         # The list to hold on to the commands we've gathered from the
         # command providers.
         gathered_commands: list[Command] = []
@@ -972,24 +1102,29 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         while hit:
             # Turn the command into something for display, and add it to the
             # list of commands that have been gathered so far.
-            prompt = hit.prompt
-            if hit.help:
-                prompt = Group(prompt, Text(hit.help, style=help_style))
+
+            def build_prompt() -> Iterable[Content]:
+                """Generator for prompt content."""
+                assert hit is not None
+                if isinstance(hit.prompt, Text):
+                    yield Content.from_rich_text(hit.prompt)
+                else:
+                    yield Content.from_markup(hit.prompt)
+
+                # Optional help text
+                if hit.help:
+                    help_style = Style.from_styles(
+                        self.get_component_styles("command-palette--help-text")
+                    )
+                    yield Content.from_markup(hit.help).stylize_before(help_style)
+
+            prompt = Content("\n").join(build_prompt())
+
             gathered_commands.append(Command(prompt, hit, id=str(command_id)))
 
-            # Before we go making any changes to the UI, we do a quick
-            # double-check that the worker hasn't been cancelled. There's
-            # little point in doing UI work on a value that isn't needed any
-            # more.
             if worker.is_cancelled:
                 break
 
-            # Having made it this far, it's safe to update the list of
-            # commands that match the input. Note that we batch up the
-            # results and only refresh the list once every so often; this
-            # helps reduce how much UI work needs to be done, but at the
-            # same time we keep the update frequency often enough so that it
-            # looks like things are moving along.
             now = monotonic()
             if (now - last_update) > self._RESULT_BATCH_TIME:
                 self._refresh_command_list(
@@ -998,7 +1133,6 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
                 clear_current = False
                 last_update = now
 
-            # Bump the ID.
             command_id += 1
 
             # Finally, get the available command from the incoming queue;
@@ -1021,8 +1155,11 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         # mean nothing was found. Give the user positive feedback to that
         # effect.
         if command_list.option_count == 0 and not worker.is_cancelled:
+            self._hit_count = 0
             self._start_no_matches_countdown(search_value)
 
+        self.add_class("-ready")
+
     def _cancel_gather_commands(self) -> None:
         """Cancel any operation that is gather commands."""
         self.workers.cancel_group(self, self._GATHER_COMMANDS_GROUP)
@@ -1051,11 +1188,13 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         input = self.query_one(CommandInput)
         with self.prevent(Input.Changed):
             assert isinstance(event.option, Command)
-            input.value = str(event.option.command.text)
-            self._selected_command = event.option.command
+            hit = event.option.hit
+            input.value = str(hit.text)
+            self._selected_command = hit
         input.action_end()
         self._list_visible = False
         self.query_one(CommandList).clear_options()
+        self._hit_count = 0
         if self.run_on_select:
             self._select_or_command()
 
@@ -1070,10 +1209,15 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
         if event is not None:
             event.stop()
         if self._list_visible:
+            command_list = self.query_one(CommandList)
             # ...so if nothing in the list is highlighted yet...
-            if self.query_one(CommandList).highlighted is None:
+            if command_list.highlighted is None:
                 # ...cause the first completion to be highlighted.
                 self._action_cursor_down()
+                # If there is one option, assume the user wants to select it
+                if command_list.option_count == 1:
+                    # Call after a short delay to provide a little visual feedback
+                    self._action_command_list("select")
             else:
                 # The list is visible, something is highlighted, the user
                 # made a selection "gesture"; let's go select it!
@@ -1085,20 +1229,22 @@ class CommandPalette(_SystemModalScreen[CallbackType]):
                 # ...we should return it to the parent screen and let it
                 # decide what to do with it (hopefully it'll run it).
                 self._cancel_gather_commands()
-                self.dismiss(self._selected_command.command)
+                self.app.post_message(CommandPalette.Closed(option_selected=True))
+                self.app.delay_update()
+                self.dismiss()
+                self.app.call_later(self._selected_command.command)
 
     @on(OptionList.OptionHighlighted)
     def _stop_event_leak(self, event: OptionList.OptionHighlighted) -> None:
         """Stop any unused events so they don't leak to the application."""
         event.stop()
+        self.app.post_message(CommandPalette.OptionHighlighted(highlighted_event=event))
 
     def _action_escape(self) -> None:
         """Handle a request to escape out of the command palette."""
-        if self._list_visible:
-            self._list_visible = False
-        else:
-            self._cancel_gather_commands()
-            self.dismiss()
+        self._cancel_gather_commands()
+        self.app.post_message(CommandPalette.Closed(option_selected=False))
+        self.dismiss()
 
     def _action_command_list(self, action: str) -> None:
         """Pass an action on to the [`CommandList`][textual.command.CommandList].
