@@ -287,7 +287,7 @@ class ScrollBar(Widget):
             background = styles.scrollbar_background
             color = styles.scrollbar_color
         if background.a < 1:
-            base_background, _ = self.parent._opacity_background_colors
+            base_background, _ = self.parent.background_colors
             background = base_background + background
         color = background + color
         scrollbar_style = Style.from_color(color.rich_color, background.rich_color)
