@@ -9,17 +9,17 @@ import rich.repr
 from rich.style import NULL_STYLE, Style
 from rich.text import Text, TextType
 
-from .. import events
-from .._immutable_sequence_view import ImmutableSequenceView
-from .._loop import loop_last
-from .._segment_tools import line_pad
-from ..binding import Binding, BindingType
-from ..cache import LRUCache
-from ..geometry import Region, Size, clamp
-from ..message import Message
-from ..reactive import reactive, var
-from ..scroll_view import ScrollView
-from ..strip import Strip
+from textual import events, on
+from textual._immutable_sequence_view import ImmutableSequenceView
+from textual._loop import loop_last
+from textual._segment_tools import line_pad
+from textual.binding import Binding, BindingType
+from textual.cache import LRUCache
+from textual.geometry import Region, Size, clamp
+from textual.message import Message
+from textual.reactive import reactive, var
+from textual.scroll_view import ScrollView
+from textual.strip import Strip
 
 if TYPE_CHECKING:
     from typing_extensions import Self, TypeAlias
@@ -50,6 +50,10 @@ class UnknownNodeID(Exception):
     """Exception raised when referring to an unknown [`TreeNode`][textual.widgets.tree.TreeNode] ID."""
 
 
+class AddNodeError(Exception):
+    """Exception raised when there is an error with a request to add a node."""
+
+
 @dataclass
 class _TreeLine(Generic[TreeDataType]):
     path: list[TreeNode[TreeDataType]]
@@ -70,13 +74,13 @@ class _TreeLine(Generic[TreeDataType]):
             Width in cells.
         """
         if show_root:
-            return 2 + (max(0, len(self.path) - 1)) * guide_depth
+            width = (max(0, len(self.path) - 1)) * guide_depth
         else:
-            guides = 2
+            width = 0
             if len(self.path) > 1:
-                guides += (len(self.path) - 1) * guide_depth
+                width += (len(self.path) - 1) * guide_depth
 
-        return guides
+        return width
 
 
 class TreeNodes(ImmutableSequenceView["TreeNode[TreeDataType]"]):
@@ -143,6 +147,14 @@ class TreeNode(Generic[TreeDataType]):
         """The child nodes of a TreeNode."""
         return TreeNodes(self._children)
 
+    @property
+    def siblings(self) -> TreeNodes[TreeDataType]:
+        """The siblings of this node (includes self)."""
+        if self.parent is None:
+            return TreeNodes([self])
+        else:
+            return self.parent.children
+
     @property
     def line(self) -> int:
         """The line number for this node, or -1 if it is not displayed."""
@@ -178,11 +190,38 @@ class TreeNode(Generic[TreeDataType]):
         """The parent of the node."""
         return self._parent
 
+    @property
+    def next_sibling(self) -> TreeNode[TreeDataType] | None:
+        """The next sibling below the node."""
+        siblings = self.siblings
+        index = siblings.index(self) + 1
+        try:
+            return siblings[index]
+        except IndexError:
+            return None
+
+    @property
+    def previous_sibling(self) -> TreeNode[TreeDataType] | None:
+        """The previous sibling below the node."""
+        siblings = self.siblings
+        index = siblings.index(self) - 1
+        if index < 0:
+            return None
+        try:
+            return siblings[index]
+        except IndexError:
+            return None
+
     @property
     def is_expanded(self) -> bool:
         """Is the node expanded?"""
         return self._expanded
 
+    @property
+    def is_collapsed(self) -> bool:
+        """Is the node collapsed?"""
+        return not self._expanded
+
     @property
     def is_last(self) -> bool:
         """Is this the last child node of its parent?"""
@@ -322,6 +361,8 @@ class TreeNode(Generic[TreeDataType]):
         label: TextType,
         data: TreeDataType | None = None,
         *,
+        before: int | TreeNode[TreeDataType] | None = None,
+        after: int | TreeNode[TreeDataType] | None = None,
         expand: bool = False,
         allow_expand: bool = True,
     ) -> TreeNode[TreeDataType]:
@@ -330,34 +371,102 @@ class TreeNode(Generic[TreeDataType]):
         Args:
             label: The new node's label.
             data: Data associated with the new node.
+            before: Optional index or `TreeNode` to add the node before.
+            after: Optional index or `TreeNode` to add the node after.
             expand: Node should be expanded.
-            allow_expand: Allow use to expand the node via keyboard or mouse.
+            allow_expand: Allow user to expand the node via keyboard or mouse.
 
         Returns:
             A new Tree node
+
+        Raises:
+            AddNodeError: If there is a problem with the addition request.
+
+        Note:
+            Only one of `before` or `after` can be provided. If both are
+            provided a `AddNodeError` will be raised.
         """
+        if before is not None and after is not None:
+            raise AddNodeError("Unable to add a node both before and after a node")
+
+        insert_index: int = len(self.children)
+
+        if before is not None:
+            if isinstance(before, int):
+                insert_index = before
+            elif isinstance(before, TreeNode):
+                try:
+                    insert_index = self.children.index(before)
+                except ValueError:
+                    raise AddNodeError(
+                        "The node specified for `before` is not a child of this node"
+                    )
+            else:
+                raise TypeError(
+                    "`before` argument must be an index or a TreeNode object to add before"
+                )
+
+        if after is not None:
+            if isinstance(after, int):
+                insert_index = after + 1
+                if after < 0:
+                    insert_index += len(self.children)
+            elif isinstance(after, TreeNode):
+                try:
+                    insert_index = self.children.index(after) + 1
+                except ValueError:
+                    raise AddNodeError(
+                        "The node specified for `after` is not a child of this node"
+                    )
+            else:
+                raise TypeError(
+                    "`after` argument must be an index or a TreeNode object to add after"
+                )
+
         text_label = self._tree.process_label(label)
         node = self._tree._add_node(self, text_label, data)
         node._expanded = expand
         node._allow_expand = allow_expand
         self._updates += 1
-        self._children.append(node)
+        self._children.insert(insert_index, node)
         self._tree._invalidate()
+
         return node
 
     def add_leaf(
-        self, label: TextType, data: TreeDataType | None = None
+        self,
+        label: TextType,
+        data: TreeDataType | None = None,
+        *,
+        before: int | TreeNode[TreeDataType] | None = None,
+        after: int | TreeNode[TreeDataType] | None = None,
     ) -> TreeNode[TreeDataType]:
         """Add a 'leaf' node (a node that can not expand).
 
         Args:
             label: Label for the node.
             data: Optional data.
+            before: Optional index or `TreeNode` to add the node before.
+            after: Optional index or `TreeNode` to add the node after.
 
         Returns:
             New node.
+
+        Raises:
+            AddNodeError: If there is a problem with the addition request.
+
+        Note:
+            Only one of `before` or `after` can be provided. If both are
+            provided a `AddNodeError` will be raised.
         """
-        node = self.add(label, data, expand=False, allow_expand=False)
+        node = self.add(
+            label,
+            data,
+            before=before,
+            after=after,
+            expand=False,
+            allow_expand=False,
+        )
         return node
 
     def _remove_children(self) -> None:
@@ -407,9 +516,36 @@ class TreeNode(Generic[TreeDataType]):
 class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     """A widget for displaying and navigating data in a tree."""
 
+    ICON_NODE = "▶ "
+    """Unicode 'icon' to use for an expandable node."""
+    ICON_NODE_EXPANDED = "▼ "
+    """Unicode 'icon' to use for an expanded node."""
+
     BINDINGS: ClassVar[list[BindingType]] = [
+        Binding("shift+left", "cursor_parent", "Cursor to parent", show=False),
+        Binding(
+            "shift+right",
+            "cursor_parent_next_sibling",
+            "Cursor to next ancestor",
+            show=False,
+        ),
+        Binding(
+            "shift+up",
+            "cursor_previous_sibling",
+            "Cursor to previous sibling",
+            show=False,
+        ),
+        Binding(
+            "shift+down",
+            "cursor_next_sibling",
+            "Cursor to next sibling",
+            show=False,
+        ),
         Binding("enter", "select_cursor", "Select", show=False),
         Binding("space", "toggle_node", "Toggle", show=False),
+        Binding(
+            "shift+space", "toggle_expand_all", "Expand or collapse all", show=False
+        ),
         Binding("up", "cursor_up", "Cursor Up", show=False),
         Binding("down", "cursor_down", "Cursor Down", show=False),
     ]
@@ -417,7 +553,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     | Key(s) | Description |
     | :- | :- |
     | enter | Select the current item. |
-    | space | Toggle the expand/collapsed space of the current item. |
+    | space | Toggle the expand/collapsed state of the current item. |
     | up | Move the cursor up. |
     | down | Move the cursor down. |
     """
@@ -445,43 +581,70 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
 
     DEFAULT_CSS = """
     Tree {
-        background: $panel;
-        color: $text;
-    }
-    Tree > .tree--label {
+        background: $surface;
+        color: $foreground;
 
-    }
-    Tree > .tree--guides {
-        color: $success-darken-3;
+        & > .tree--label {}
+        & > .tree--guides {
+            color: $surface-lighten-2;
+        }
+        & > .tree--guides-hover {
+            color: $surface-lighten-2;
+        }
+        & > .tree--guides-selected {
+            color: $block-cursor-blurred-background;
+        }
+        & > .tree--cursor {
+            text-style: $block-cursor-blurred-text-style;
+            background: $block-cursor-blurred-background;
+        }
+        & > .tree--highlight {}
+        & > .tree--highlight-line {
+            background: $block-hover-background;
+        }
+
+        &:focus {
+            background-tint: $foreground 5%;
+            & > .tree--cursor {
+                color: $block-cursor-foreground;
+                background: $block-cursor-background;
+                text-style: $block-cursor-text-style;
+            }
+            & > .tree--guides {
+                color: $surface-lighten-3;
+            }
+            & > .tree--guides-hover {
+                color: $surface-lighten-3;
+            }
+            & > .tree--guides-selected {
+                color: $block-cursor-background;
+            }
+        }
+
+        &:light {
+            /* In light mode the guides are darker*/
+            & > .tree--guides {
+                color: $surface-darken-1;
+            }
+            & > .tree--guides-hover {
+                color: $block-cursor-background;
+            }
+            & > .tree--guides-selected {
+                color: $block-cursor-background;
+            }
+        }
+
+        &:ansi {
+            color: ansi_default;
+            & > .tree--guides {
+                color: ansi_green;
+            }
+            &:nocolor > .tree--cursor{
+                text-style: reverse;
+            }
+        }
     }
 
-    Tree > .tree--guides-hover {
-        color: $success;
-        text-style: bold;
-    }
-
-    Tree > .tree--guides-selected {
-        color: $warning;
-        text-style: bold;
-    }
-
-    Tree > .tree--cursor {
-        background: $secondary-darken-2;
-        color: $text;
-        text-style: bold;
-    }
-
-    Tree:focus > .tree--cursor {
-        background: $secondary;
-    }
-
-    Tree > .tree--highlight {
-        text-style: underline;
-    }
-
-    Tree > .tree--highlight-line {
-        background: $boost;
-    }
     """
 
     show_root = reactive(True)
@@ -495,7 +658,9 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     guide_depth = reactive(4, init=False)
     """The indent depth of tree nodes."""
     auto_expand = var(True)
-    """Auto expand tree nodes when clicked."""
+    """Auto expand tree nodes when they are selected."""
+    center_scroll = var(False)
+    """Keep selected node in the center of the control, where possible."""
 
     LINES: dict[str, tuple[str, str, str, str]] = {
         "default": (
@@ -620,6 +785,52 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
 
         super().__init__(name=name, id=id, classes=classes, disabled=disabled)
 
+    def add_json(self, json_data: object, node: TreeNode | None = None) -> None:
+        """Adds JSON data to a node.
+
+        Args:
+            json_data: An object decoded from JSON.
+            node: Node to add data to.
+
+        """
+
+        if node is None:
+            node = self.root
+
+        from rich.highlighter import ReprHighlighter
+
+        highlighter = ReprHighlighter()
+
+        def add_node(name: str, node: TreeNode, data: object) -> None:
+            """Adds a node to the tree.
+
+            Args:
+                name: Name of the node.
+                node: Parent node.
+                data: Data associated with the node.
+            """
+            if isinstance(data, dict):
+                node.set_label(Text(f"{{}} {name}"))
+                for key, value in data.items():
+                    new_node = node.add("")
+                    add_node(key, new_node, value)
+            elif isinstance(data, list):
+                node.set_label(Text(f"[] {name}"))
+                for index, value in enumerate(data):
+                    new_node = node.add("")
+                    add_node(str(index), new_node, value)
+            else:
+                node.allow_expand = False
+                if name:
+                    label = Text.assemble(
+                        Text.from_markup(f"[b]{name}[/b]="), highlighter(repr(data))
+                    )
+                else:
+                    label = Text(repr(data))
+                node.set_label(label)
+
+        add_node("", node, json_data)
+
     @property
     def cursor_node(self) -> TreeNode[TreeDataType] | None:
         """The currently selected node, or ``None`` if no selection."""
@@ -633,7 +844,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     def process_label(self, label: TextType) -> Text:
         """Process a `str` or `Text` value into a label.
 
-        Maybe overridden in a subclass to change how labels are rendered.
+        May be overridden in a subclass to change how labels are rendered.
 
         Args:
             label: Label.
@@ -678,7 +889,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
 
         if node._allow_expand:
             prefix = (
-                "▼ " if node.is_expanded else "▶ ",
+                self.ICON_NODE_EXPANDED if node.is_expanded else self.ICON_NODE,
                 base_style + TOGGLE_STYLE,
             )
         else:
@@ -690,7 +901,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
     def get_label_width(self, node: TreeNode[TreeDataType]) -> int:
         """Get the width of the nodes label.
 
-        The default behavior is to call `render_node` and return the cell length. This method may be
+        The default behavior is to call `render_label` and return the cell length. This method may be
         overridden in a sub-class if it can be done more efficiently.
 
         Args:
@@ -745,13 +956,62 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         self.root.data = data
         return self
 
-    def select_node(self, node: TreeNode[TreeDataType] | None) -> None:
+    def move_cursor(
+        self, node: TreeNode[TreeDataType] | None, animate: bool = False
+    ) -> None:
         """Move the cursor to the given node, or reset cursor.
 
         Args:
             node: A tree node, or None to reset cursor.
+            animate: Enable animation
         """
+        previous_cursor_line = self.cursor_line
         self.cursor_line = -1 if node is None else node._line
+        if node is not None and self.cursor_node is not None:
+            self.scroll_to_node(
+                self.cursor_node,
+                animate=animate and abs(self.cursor_line - previous_cursor_line) > 1,
+            )
+
+    def move_cursor_to_line(self, line: int, animate=False) -> None:
+        """Move the cursor to the given line.
+
+        Args:
+            line: The line number (negative indexes are offsets from the last line).
+            animate: Enable scrolling animation.
+
+        Raises:
+            IndexError: If the line doesn't exist.
+        """
+        if self.cursor_line == line:
+            return
+        try:
+            node = self._tree_lines[line].node
+        except IndexError:
+            raise IndexError(f"No line no. {line} in the tree")
+        self.move_cursor(node, animate=animate)
+
+    def select_node(self, node: TreeNode[TreeDataType] | None) -> None:
+        """Move the cursor to the given node and select it, or reset cursor.
+
+        Args:
+            node: A tree node to move the cursor to and select, or None to reset cursor.
+        """
+        self.move_cursor(node)
+        if node is not None:
+            self.post_message(Tree.NodeSelected(node))
+
+    def unselect(self) -> None:
+        """Hide and reset the cursor."""
+        self.set_reactive(Tree.cursor_line, -1)
+        self._invalidate()
+
+    @on(NodeSelected)
+    def _expand_node_on_select(self, event: NodeSelected[TreeDataType]) -> None:
+        """When the node is selected, expand the node if `auto_expand` is True."""
+        node = event.node
+        if self.auto_expand:
+            self._toggle_node(node)
 
     def get_node_at_line(self, line_no: int) -> TreeNode[TreeDataType] | None:
         """Get the node for a given line.
@@ -822,6 +1082,10 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         else:
             self.hover_line = -1
 
+    def _on_leave(self, _: events.Leave) -> None:
+        # Ensure the hover effect doesn't linger after the mouse leaves.
+        self.hover_line = -1
+
     def _new_id(self) -> NodeID:
         """Create a new node ID.
 
@@ -833,6 +1097,8 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         return NodeID(id)
 
     def _get_node(self, line: int) -> TreeNode[TreeDataType] | None:
+        if line == -1:
+            return None
         try:
             tree_line = self._tree_lines[line]
         except IndexError:
@@ -874,21 +1140,34 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
 
     def watch_cursor_line(self, previous_line: int, line: int) -> None:
         previous_node = self._get_node(previous_line)
-        if previous_node is not None:
-            self._refresh_node(previous_node)
-            previous_node._selected = False
-            self._cursor_node = None
-
         node = self._get_node(line)
+
+        if self.cursor_node is not None:
+            self.cursor_node._selected = False
+
+        if previous_node is not None:
+            previous_node._selected = False
+
         if node is not None:
-            self._refresh_node(node)
             node._selected = True
             self._cursor_node = node
-            if previous_node != node:
-                self.post_message(self.NodeHighlighted(node))
         else:
             self._cursor_node = None
 
+        if previous_line == line:
+            # No change, so no need for refresh
+            return
+
+        # Refresh previous cursor node
+        if previous_node is not None:
+            self._refresh_node(previous_node)
+
+        # Refresh new node
+        if node is not None:
+            self._refresh_node(node)
+            if previous_node != node:
+                self.post_message(self.NodeHighlighted(node))
+
     def watch_guide_depth(self, guide_depth: int) -> None:
         self._invalidate()
 
@@ -905,7 +1184,14 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         """
         region = self._get_label_region(line)
         if region is not None:
-            self.scroll_to_region(region, animate=animate, force=True)
+            self.scroll_to_region(
+                region,
+                animate=animate,
+                force=True,
+                center=self.center_scroll,
+                origin_visible=False,
+                x_axis=False,  # Scrolling the X axis is quite jarring, and rarely necessary
+            )
 
     def scroll_to_node(
         self, node: TreeNode[TreeDataType], animate: bool = True
@@ -913,7 +1199,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         """Scroll to the given node.
 
         Args:
-            node: Node to scroll in to view.
+            node: Node to scroll into view.
             animate: Animate scrolling.
         """
         line = node._line
@@ -1006,7 +1292,6 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
                 self.cursor_line = self.cursor_node._line
             if self.cursor_line >= len(lines):
                 self.cursor_line = -1
-        self.refresh()
 
     def render_lines(self, crop: Region) -> list[Strip]:
         self._pseudo_class_state = self.get_pseudo_class_state()
@@ -1045,6 +1330,13 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
         if cache_key in self._line_cache:
             strip = self._line_cache[cache_key]
         else:
+            # Allow tree guides to be explicitly disabled by setting color to transparent
+            base_hidden = self.get_component_styles("tree--guides").color.a == 0
+            hover_hidden = self.get_component_styles("tree--guides-hover").color.a == 0
+            selected_hidden = (
+                self.get_component_styles("tree--guides-selected").color.a == 0
+            )
+
             base_guide_style = self.get_component_rich_style(
                 "tree--guides", partial=True
             )
@@ -1058,17 +1350,18 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
             hover = line.path[0]._hover
             selected = line.path[0]._selected and self.has_focus
 
-            def get_guides(style: Style) -> tuple[str, str, str, str]:
+            def get_guides(style: Style, hidden: bool) -> tuple[str, str, str, str]:
                 """Get the guide strings for a given style.
 
                 Args:
                     style: A Style object.
+                    hidden: Switch to hide guides (make them invisible).
 
                 Returns:
                     Strings for space, vertical, terminator and cross.
                 """
                 lines: tuple[Iterable[str], Iterable[str], Iterable[str], Iterable[str]]
-                if self.show_guides:
+                if self.show_guides and not hidden:
                     lines = self.LINES["default"]
                     if style.bold:
                         lines = self.LINES["bold"]
@@ -1089,17 +1382,24 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
             else:
                 line_style = base_style
 
+            line_style += Style(meta={"line": y})
+
             guides = Text(style=line_style)
             guides_append = guides.append
 
             guide_style = base_guide_style
+
+            hidden = True
             for node in line.path[1:]:
+                hidden = base_hidden
                 if hover:
                     guide_style = guide_hover_style
+                    hidden = hover_hidden
                 if selected:
                     guide_style = guide_selected_style
+                    hidden = selected_hidden
 
-                space, vertical, _, _ = get_guides(guide_style)
+                space, vertical, _, _ = get_guides(guide_style, hidden)
                 guide = space if node.is_last else vertical
                 if node != line.path[-1]:
                     guides_append(guide, style=guide_style)
@@ -1107,7 +1407,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
                 selected = (selected or node._selected) and self.has_focus
 
             if len(line.path) > 1:
-                _, _, terminator, cross = get_guides(guide_style)
+                _, _, terminator, cross = get_guides(guide_style, hidden)
                 if line.last:
                     guides.append(terminator, style=guide_style)
                 else:
@@ -1124,7 +1424,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
                 )
 
             label = self.render_label(line.path[-1], line_style, label_style).copy()
-            label.stylize(Style(meta={"node": line.node._id, "line": y}))
+            label.stylize(Style(meta={"node": line.node._id}))
             guides.append(label)
 
             segments = list(guides.render(self.app.console))
@@ -1162,6 +1462,7 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
                     await self.run_action("select_cursor")
 
     def notify_style_update(self) -> None:
+        super().notify_style_update()
         self._invalidate()
 
     def action_cursor_up(self) -> None:
@@ -1225,12 +1526,73 @@ class Tree(Generic[TreeDataType], ScrollView, can_focus=True):
             will cause both an expand/collapse event to occur, as well as a
             selected event.
         """
+        if self.cursor_line < 0:
+            return
         try:
             line = self._tree_lines[self.cursor_line]
         except IndexError:
             pass
         else:
             node = line.path[-1]
-            if self.auto_expand:
-                self._toggle_node(node)
-            self.post_message(self.NodeSelected(node))
+            self.post_message(Tree.NodeSelected(node))
+
+    def action_cursor_parent(self) -> None:
+        """Move the cursor to the parent node."""
+        cursor_node = self.cursor_node
+        if cursor_node is not None and cursor_node.parent is not None:
+            self.move_cursor(cursor_node.parent, animate=True)
+
+    def action_cursor_parent_next_sibling(self) -> None:
+        """Move the cursor to the parent's next sibling."""
+        cursor_node = self.cursor_node
+        if cursor_node is not None and cursor_node.parent is not None:
+            self.move_cursor(cursor_node.parent.next_sibling, animate=True)
+
+    def action_cursor_previous_sibling(self) -> None:
+        """Move the cursor to previous sibling, or to the parent if there are no more siblings."""
+        cursor_node = self.cursor_node
+        if cursor_node is not None:
+            previous_sibling = cursor_node.previous_sibling
+            if previous_sibling is None:
+                self.move_cursor(cursor_node.parent, animate=True)
+            else:
+                self.move_cursor(previous_sibling, animate=True)
+
+    def action_cursor_next_sibling(self) -> None:
+        """Move the cursor to the next sibling, or to the paren't sibling if there are no more siblings."""
+        cursor_node = self.cursor_node
+        if cursor_node is not None:
+            next_sibling = cursor_node.next_sibling
+            if next_sibling is None:
+                if cursor_node.parent is not None:
+                    parent_sibling = cursor_node.parent.next_sibling
+                    self.move_cursor(parent_sibling, animate=True)
+            else:
+                self.move_cursor(next_sibling, animate=True)
+
+    def action_toggle_expand_all(self) -> None:
+        """Expand or collapse all siblings.
+
+        If all the siblings are collapsed then they will be expanded.
+        Otherwise they will all be collapsed.
+
+        """
+
+        if self.cursor_node is None or self.cursor_node.parent is None:
+            return
+
+        siblings = self.cursor_node.siblings
+        cursor_node = self.cursor_node
+
+        # If all siblings are collapsed we want to expand them all
+        if all(child.is_collapsed for child in siblings):
+            for child in siblings:
+                if child.allow_expand:
+                    child.expand()
+        # Otherwise we want to collapse them all
+        else:
+            for child in siblings:
+                if child.allow_expand:
+                    child.collapse()
+
+        self.call_after_refresh(self.move_cursor, cursor_node, animate=False)
